// Copyright (c) 2017, Tobias Mueller tm(at)tm3d.de
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the
//    distribution.
//  * All advertising materials mentioning features or use of this
//    software must display the following acknowledgement: This product
//    includes software developed by tm3d.de and its contributors.
//  * Neither the name of tm3d.de nor the names of its contributors may
//    be used to endorse or promote products derived from this software
//    without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "mySensorDB.h"
#include <iostream>
#include <algorithm>
#include <fstream>
#include <sstream> 
mySensorDB::mySensorDB(std::string configfile, owLog* log) {
	this->log=log;
	log->clear();
	my=0;
	readConfigFile(configfile);
	if (log->last()<OWLOG_ERROR) {
		my = mysql_init(NULL);
		if (my == NULL) {
			log->set(OWLOG_ERROR,"ERROR: Could not initialize Mysql");
		}
	}
}

int mySensorDB::readConfigFile(std::string configfile) {
	std::ifstream dbc(configfile);
	if (!dbc) {
		log->set(OWLOG_ERROR,"Error reading database config file %s",configfile.c_str());
		return -1;
	}
	int lc=0;
	std::string line;
	while (std::getline(dbc, line)) {
		switch (lc) {
			case 0:myserver=line;break;
			case 1:myport=line;break;
			case 2:myuser=line;break;
			case 3:mypass=line;break;
			case 4:mydb=line;break;
			case 5:mypre=line;break;
		}
		lc++;
		if (lc==6) break;
	}
	if (lc<6) {
		log->set(OWLOG_ERROR,"ERROR: Database config file isn't correct. (not 6 lines)\n");
		return -2;
	}
	return 0;
}


int mySensorDB::connect() {
	if( mysql_real_connect (
			my,   /* Zeiger auf MYSQL-Handler */
			myserver.c_str(), /* Host-Name */
			myuser.c_str(), /* User-Name */
			mypass.c_str(), /* Passwort für user_name */
			mydb.c_str(),  /* Name der Datenbank */
			atol(myport.c_str()),     /* Port (default=0) */
			NULL,  /* Socket (default=NULL) */
			0      /* keine Flags */  )  == NULL) {

		log->set(OWLOG_ERROR,"ERROR mysql_real_connect(): %u (%s)\n",mysql_errno (my), mysql_error (my));
		return -1;
	}
	printf("DB ok\n");
	return 0;
}

int mySensorDB::createSensorTable() {
	std::string query="CREATE TABLE IF NOT EXISTS `"+mypre+"sensors` ("
  							"  `sid` varchar(20) NOT NULL COMMENT 'owID + number of 1 to 4' ,"
							 " `sdescription` varchar(128) DEFAULT NULL,"
							 " `schip` varchar(120) DEFAULT NULL COMMENT 'Chip of mesasurement',"
							 " `smaxval` double DEFAULT NULL COMMENT 'Maximal value',"
							"  `sminval` double DEFAULT NULL COMMENT 'Minimal value',"
							 " `slocation` varchar(60) DEFAULT NULL COMMENT 'Position of Sensor, if needed',"
							 " `stype` varchar(40) DEFAULT NULL COMMENT 'Name of pysical property',"
							 " `sunit` varchar(20) DEFAULT NULL COMMENT 'Physical unit',"
							 " `sprecision` double DEFAULT NULL COMMENT '+- precission',"
							 " `sprecisionlong` text DEFAULT NULL COMMENT 'a table of specific precission values',"
							 " `sfirstuse` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,"
							 " `scalib_m` double DEFAULT '0' COMMENT 'an optional addition value',"
							 " `scalib_n` double DEFAULT '1' COMMENT 'an optimal multip. value', "
							 " `shidden` tinyint(1) DEFAULT '0' COMMENT 'for automatic presentation',"
							 " `sformelcode` tinyint(1) DEFAULT '0' COMMENT 'code of the formel',"
							 " `stypecode` tinyint(1) DEFAULT '0' COMMENT 'code of the property',"
							 " PRIMARY KEY (`sid`)"
							") DEFAULT CHARSET=utf8;"; 
						
	if (mysql_query(my,query.c_str()	))	{
		log->set(OWLOG_ERROR, "ERROR mysql_query: %u (%s)\n",mysql_errno (my), mysql_error (my));
		return -1;
	}
	return 0;
}



int mySensorDB::createDeviceTable(owDevice *dev) {
	if (dev->config->valid) {
		std::string s;
		char hs[50];
		s="SELECT sid FROM "+mypre+"sensors WHERE sid='";
		sprintf(hs,"%016llX",(unsigned long long)dev->getNum().num);
		s=s+hs+"_1';";
		if (mysql_query(my,s.c_str())) {
			//printf("ERROR communicating Database\n");
			log->set(OWLOG_ERROR,"ERROR mysql_query: %u (%s)\n",mysql_errno (my), mysql_error (my));
			return -1;
		}  
		MYSQL_RES *result = mysql_store_result(my);
		if (result == NULL)  {
			//finish_with_error(my);
			log->set(OWLOG_ERROR,"ERROR get result from Database\n");
			return -1;
		} 
		if (mysql_num_rows(result)==0) {
			log->set(OWLOG_INFO,"Sensor not in Database\n");
			std::string cols;
			cols="CREATE TABLE IF NOT EXISTS `"+mypre+"OW"+std::string(hs)+"` (`time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP ";
			for (size_t i=0;i<4;i++)  {
				if (dev->config->getPropertyID(i)!=0) {
					cols+=", `";
					cols+=std::string(hs)+"_"+std::to_string(i+1);
					cols+="` double DEFAULT NULL ";
					std::string query;
					query="INSERT INTO "+mypre+"sensors (sid,schip,stype,sunit,sformelcode,stypecode) VALUES ('"+std::string(hs)+"_"+std::to_string(i+1)+"' ";
					query+=", '"+dev->config->getSensorChip(i)+"' , '";
					query+=dev->config->getQuantity(i)+"' , '";
					query+=dev->config->getUnit(i)+"' ," ;
					query+=std::to_string(dev->config->getFormulaID(i))+" ,";
					query+=std::to_string(dev->config->getPropertyID(i))+");";
					//printf("%s\n",query.c_str());
					if (mysql_query(my,query.c_str())) {
						log->set(OWLOG_ERROR, "ERROR mysql_query: %u (%s)\n",mysql_errno (my), mysql_error (my));
						return -1;
					}
				}
			}
			cols+=", PRIMARY KEY (`time`) ) DEFAULT CHARSET=utf8;";
			//printf("%s\n",cols.c_str());
			if (mysql_query(my,cols.c_str())) {
			log->set(OWLOG_ERROR, "ERROR mysql_query: %u (%s)\n",mysql_errno (my), mysql_error (my));
				return -1;
			}
	
		} 
				//int num_fields=mysql_num_fields(result)
				//MYSQL_ROW row;
				//while ((row = mysql_fetch_row(result))) {
				//  for(int i = 0; i < num_fields; i++) { 
				//		printf("%s ", row[i] ? row[i] : "NULL"); 
				//	} 
				//	printf("\n"); 
				//}
		mysql_free_result(result);
	}
	return 0;
}

int mySensorDB::insertValues(owDevice *dev) {
	if (dev->config->valid) {
		char hs[50];
		sprintf(hs,"%016llX",(unsigned long long)dev->getNum().num);
		std::string query;
		query="INSERT INTO `"+mypre+"OW"+std::string(hs)+"` (";
		for (size_t i=0;i<4;i++) if (dev->config->getPropertyID(i)!=0) query+=std::string(hs)+"_"+std::to_string(i+1)+", ";
		query.pop_back();query.pop_back();
		query+=") VALUES (";
		for (size_t i=0;i<4;i++) if (dev->config->getPropertyID(i)!=0) query+=std::to_string(dev->values[i])+", ";
		query.pop_back();query.pop_back();
		query+=");";
		//printf("%s\n",query.c_str());
		if (mysql_query(my,query.c_str())) {
			log->set(OWLOG_ERROR, "ERROR mysql_query: %u (%s)\n",mysql_errno (my), mysql_error (my));
			return -1;
		}
	}
	
	return 0;
}
